'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var styled = _interopDefault(require('styled-components'));

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends.apply(this, arguments);
}

function _taggedTemplateLiteralLoose(strings, raw) {
  if (!raw) {
    raw = strings.slice(0);
  }

  strings.raw = raw;
  return strings;
}

var _templateObject, _templateObject2, _templateObject3, _templateObject4, _templateObject5, _templateObject6, _templateObject8, _templateObject9;
var Container = /*#__PURE__*/styled.div(_templateObject || (_templateObject = /*#__PURE__*/_taggedTemplateLiteralLoose(["\n  position: relative;\n  width: 100%;\n  height: 100%;\n  overflow: hidden;\n"])));
var Scrollable = /*#__PURE__*/styled.div(_templateObject2 || (_templateObject2 = /*#__PURE__*/_taggedTemplateLiteralLoose(["\n  ::-webkit-scrollbar {\n    display: none;\n  }\n  position: absolute;\n  top: 0;\n  left: 0;\n  height: 100%;\n  width: 100%;\n  overflow: scroll !important;\n  scroll-padding: 100px 0 0 100px;\n  -ms-overflow-style: none;\n  scrollbar-width: none;\n"])));
var HorizontalSpacer = /*#__PURE__*/styled.div(_templateObject3 || (_templateObject3 = /*#__PURE__*/_taggedTemplateLiteralLoose(["\n  position: relative;\n  height: 50%;\n  width: 100%;\n"])));
var RowContainer = /*#__PURE__*/styled.div(_templateObject4 || (_templateObject4 = /*#__PURE__*/_taggedTemplateLiteralLoose(["\n  display: flex;\n  flex-direction: column;\n  justify-content: center;\n  align-items: center;\n  padding-left: 50%;\n  padding-right: 50%;\n  box-sizing: content-box;\n"])));
var Row = /*#__PURE__*/styled.div(_templateObject5 || (_templateObject5 = /*#__PURE__*/_taggedTemplateLiteralLoose(["\n  display: flex;\n  flex-direction: row;\n"])));
var BubbleContainer = /*#__PURE__*/styled.div(_templateObject6 || (_templateObject6 = /*#__PURE__*/_taggedTemplateLiteralLoose(["\n  display: flex;\n  justify-content: center;\n  align-items: center;\n"])));
var GuideContainer = /*#__PURE__*/styled.div(_templateObject8 || (_templateObject8 = /*#__PURE__*/_taggedTemplateLiteralLoose(["\n  position: absolute;\n  top: 0;\n  left: 0;\n  height: 100%;\n  width: 100%;\n  pointer-events: none;\n"])));
var Guide = /*#__PURE__*/styled.div(_templateObject9 || (_templateObject9 = /*#__PURE__*/_taggedTemplateLiteralLoose(["\n  position: absolute;\n  top: 50%;\n  left: 50%;\n  transform: translate(-50%, -50%);\n  border: 2px solid rgba(0, 0, 0, 0.1);\n  background-color: rgba(0, 0, 0, 0.12);\n"])));

var defaultOptions = {
  size: 200,
  minSize: 20,
  gutter: 16,
  provideProps: false,
  numCols: 6,
  fringeWidth: 100,
  yRadius: 200,
  xRadius: 200,
  cornerRadius: 100,
  showGuides: false,
  compact: false,
  gravitation: 0
};
function Bubble(props) {
  var _useState = React.useState(0),
      scrollTop = _useState[0],
      setScrollTop = _useState[1];

  var _useState2 = React.useState(0),
      scrollLeft = _useState2[0],
      setScrollLeft = _useState2[1];

  var scrollable = React.useRef(null);

  var handleScroll = function handleScroll(e) {
    if (e.target.className) {
      setScrollTop(e.target.scrollTop);
      setScrollLeft(e.target.scrollLeft);
    }
  };

  React.useLayoutEffect(function () {
    window.addEventListener('scroll', handleScroll);

    if (scrollable.current) {
      window.scrollTo((scrollable.current.scrollWidth - scrollable.current.clientWidth) / 2, (scrollable.current.scrollHeight - scrollable.current.clientHeight) / 2);
    }

    return function () {
      return window.removeEventListener('scroll', handleScroll);
    };
  }, []);

  var options = _extends({}, defaultOptions);

  Object.assign(options, defaultOptions);
  Object.assign(options, props.options);
  options.numCols = Math.min(options.numCols, React.Children.toArray(props.children).length);
  var minProportion = options.minSize / options.size;
  var verticalPadding = "calc(50% - " + (options.yRadius + options.size / 2 - options.cornerRadius * (1.414 - 1) / 1.414) + "px)";
  var horizontalPadding = "calc(50% - " + (options.xRadius + options.size / 2 - options.cornerRadius * (1.414 - 1) / 1.414) + "px)";
  var rows = [];
  var colsRemaining = 0;
  var evenRow = true;

  for (var i = 0; i < React.Children.toArray(props.children).length; i++) {
    var _props$children;

    if (colsRemaining === 0) {
      colsRemaining = evenRow ? options.numCols - 1 : options.numCols;
      evenRow = !evenRow;
      rows.push([]);
    }

    rows[rows.length - 1].push(props == null ? void 0 : (_props$children = props.children) == null ? void 0 : _props$children[i]);
    colsRemaining--;
  }

  if (rows.length > 1) {
    if (rows[rows.length - 1].length % 2 === rows[rows.length - 2].length % 2) {
      rows[rows.length - 1].push(React__default.createElement("div", null)); // dummy bubble
    }
  }

  var interpolate = function interpolate(actualMin, actualMax, val, targetMin, targetMax) {
    return (val - actualMin) / (actualMax - actualMin) * (targetMax - targetMin) + targetMin;
  };

  var getBubbleSize = function getBubbleSize(row, col) {
    var yOffset = (options.size + options.gutter) * 0.866 * row - options.size + options.cornerRadius * (1.414 - 1) / 1.414 - (options.yRadius - options.size);
    var xOffset = (options.size + options.gutter) * col + (options.numCols - rows[row].length) * (options.size + options.gutter) / 2 - options.size + options.cornerRadius * (1.414 - 1) / 1.414 - (options.xRadius - options.size);
    var dy = yOffset - scrollTop;
    var dx = xOffset - scrollLeft;
    var distance = Math.sqrt(dx * dx + dy * dy);
    var out = {
      bubbleSize: 1,
      translateX: 0,
      translateY: 0,
      distance: distance
    };
    var distanceFromEdge = 0;
    var isInCornerRegion = false;

    if (Math.abs(dx) <= options.xRadius && Math.abs(dy) <= options.yRadius) {
      // inner square
      if (Math.abs(dy) > options.yRadius - options.cornerRadius && Math.abs(dx) > options.xRadius - options.cornerRadius) {
        // in corner region
        var distToInnerCorner = Math.sqrt(Math.pow(Math.abs(dy) - options.yRadius + options.cornerRadius, 2) + Math.pow(Math.abs(dx) - options.xRadius + options.cornerRadius, 2));

        if (distToInnerCorner > options.cornerRadius) {
          // outside inner radius
          distanceFromEdge = distToInnerCorner - options.cornerRadius;
          isInCornerRegion = true;
        }
      }
    } else if (Math.abs(dx) <= options.xRadius + options.fringeWidth && Math.abs(dy) <= options.yRadius + options.fringeWidth) {
      // outer square
      if (Math.abs(dy) > options.yRadius - options.cornerRadius && Math.abs(dx) > options.xRadius - options.cornerRadius) {
        // in corner region
        isInCornerRegion = true;

        var _distToInnerCorner = Math.sqrt(Math.pow(Math.abs(dy) - options.yRadius + options.cornerRadius, 2) + Math.pow(Math.abs(dx) - options.xRadius + options.cornerRadius, 2));

        distanceFromEdge = _distToInnerCorner - options.cornerRadius;
      } else {
        distanceFromEdge = Math.max(Math.abs(dx) - options.xRadius, Math.abs(dy) - options.yRadius);
      }
    } else {
      // outside outer square
      isInCornerRegion = Math.abs(dy) > options.yRadius - options.cornerRadius && Math.abs(dx) > options.xRadius - options.cornerRadius;

      if (isInCornerRegion) {
        var _distToInnerCorner2 = Math.sqrt(Math.pow(Math.abs(dy) - options.yRadius + options.cornerRadius, 2) + Math.pow(Math.abs(dx) - options.xRadius + options.cornerRadius, 2));

        distanceFromEdge = _distToInnerCorner2 - options.cornerRadius;
      } else {
        distanceFromEdge = Math.max(Math.abs(dx) - options.xRadius, Math.abs(dy) - options.yRadius);
      }
    }

    out.bubbleSize = interpolate(0, options.fringeWidth, Math.min(distanceFromEdge, options.fringeWidth), 1, minProportion); //handle magnitudes

    var translationMag = options.compact ? (options.size - options.minSize) / 2 : 0;
    var interpolatedTranslationMag = interpolate(0, options.fringeWidth, distanceFromEdge, 0, translationMag);

    if (distanceFromEdge > 0 && distanceFromEdge <= options.fringeWidth) {
      out.translateX = interpolatedTranslationMag;
      out.translateY = interpolatedTranslationMag;
    } else if (distanceFromEdge - options.fringeWidth > 0) {
      var extra = Math.max(0, distanceFromEdge - options.fringeWidth - options.size / 2) * options.gravitation / 10;
      out.translateX = translationMag + extra;
      out.translateY = translationMag + extra;
    }

    if (isInCornerRegion) {
      var cornerDx = Math.abs(dx) - options.xRadius + options.cornerRadius;
      var cornerDy = Math.abs(dy) - options.yRadius + options.cornerRadius;
      var theta = Math.atan(-cornerDy / cornerDx);

      if (dx > 0) {
        if (dy > 0) {
          theta *= -1;
        }
      } else {
        if (dy > 0) {
          theta += Math.PI;
        } else {
          theta += Math.PI - 2 * theta;
        }
      }

      out.translateX *= -Math.cos(theta);
      out.translateY *= -Math.sin(theta);
    } else if (Math.abs(dx) > options.xRadius || Math.abs(dy) > options.yRadius) {
      if (Math.abs(dx) > options.xRadius) {
        out.translateX *= -Math.sign(dx);
        out.translateY = 0;
      } else {
        out.translateY *= -Math.sign(dy);
        out.translateX = 0;
      }
    }

    return out;
  };

  return React__default.createElement("div", {
    className: props.className,
    style: {
      display: 'flex',
      justifyContent: 'center',
      alignItems: 'center'
    }
  }, React__default.createElement(Container, null, React__default.createElement(Scrollable, {
    ref: scrollable,
    onScroll: handleScroll
  }, React__default.createElement(HorizontalSpacer, {
    style: {
      height: verticalPadding
    }
  }), React__default.createElement(RowContainer, {
    style: {
      width: options.size * options.numCols + options.gutter * (options.numCols - 1),
      paddingLeft: horizontalPadding,
      paddingRight: horizontalPadding
    }
  }, rows.map(function (row, i) {
    return React__default.createElement(Row, {
      key: row + "-" + i,
      style: {
        marginTop: i > 0 ? options.size * -0.134 + options.gutter * 0.866 // .134 is sqrt(3) - 1
        : 0
      }
    }, row.map(function (comp, j) {
      var _getBubbleSize = getBubbleSize(i, j),
          bubbleSize = _getBubbleSize.bubbleSize,
          translateX = _getBubbleSize.translateX,
          translateY = _getBubbleSize.translateY,
          distance = _getBubbleSize.distance;

      return React__default.createElement(BubbleContainer, {
        key: comp + "-" + j,
        style: {
          width: options.size,
          height: options.size,
          marginRight: options.gutter / 2,
          marginLeft: options.gutter / 2,
          transform: "translateX(" + translateX + "px) translateY(" + translateY + "px) scale(" + bubbleSize + ")"
        }
      }, options.provideProps ? React__default.cloneElement(comp, {
        bubbleSize: bubbleSize * options.size,
        distanceToCenter: distance,
        maxSize: options.size,
        minSize: options.minSize
      }) : comp);
    }));
  })), React__default.createElement(HorizontalSpacer, {
    style: {
      height: verticalPadding
    }
  })), options.showGuides ? React__default.createElement(GuideContainer, null, React__default.createElement(Guide, {
    style: {
      height: options.yRadius * 2,
      width: options.xRadius * 2,
      borderRadius: options.shape === 'ellipse' ? '50%' : options.cornerRadius
    }
  }), React__default.createElement(Guide, {
    style: {
      height: (options.yRadius + options.fringeWidth) * 2,
      width: (options.xRadius + options.fringeWidth) * 2,
      borderRadius: options.shape === 'ellipse' ? '50%' : options.cornerRadius + options.fringeWidth
    }
  })) : null));
}

exports.Bubble = Bubble;
exports.defaultOptions = defaultOptions;
//# sourceMappingURL=react-bubble-ui-v2.cjs.development.js.map
